@@ -1,9 +1,6 @@
 /* -*- mode:objc; coding:utf-8; tab-width:8; c-basic-offset:2; indent-tabs-mode:nil -*- */
 /*
-  Copyright (c) 2003-2004 MacUIM contributors and others
-  http://www.digital-genes.com/~yatsu/macuim/
-
-  All rights reserved.
+  Copyright (c) 2003-2005 MacUIM contributors, All rights reserved.
 
   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions
@@ -37,7 +34,7 @@ static CocoaWinController *sharedController;
 
 @implementation CocoaWinController
  
-+ (CocoaWinController *)sharedController
++ (id)sharedController
 {
   return sharedController;
 }
@@ -181,21 +178,15 @@ static CocoaWinController *sharedController;
    forTableColumn:(NSTableColumn *)tableColumn
               row:(int)row
 {
+  /*
   [cell setDrawsBackground:YES];
   
-  /*
-  if ([[tableColumn identifier] isEqual:@"head"]) {
-    [cell setBackgroundColor:[NSColor darkGrayColor]];
-    [cell setTextColor:[NSColor whiteColor]];
-  }
+  if (row % 2)
+    [cell setBackgroundColor:[NSColor colorWithCalibratedWhite:0.95
+                                                         alpha:1.0]];
+  else
+    [cell setBackgroundColor:[NSColor whiteColor]];
   */
-  //else {
-    if (row % 2)
-      [cell setBackgroundColor:[NSColor colorWithCalibratedWhite:0.95
-                                                           alpha:1.0]];
-    else
-      [cell setBackgroundColor:[NSColor whiteColor]];
-  //}
   
   [cell setFont:font];
 }
