@@ -1,9 +1,6 @@
 /* -*- mode:c; coding:utf-8; tab-width:8; c-basic-offset:2; indent-tabs-mode:nil -*- */
 /*
-  Copyright (c) 2003-2004 MacUIM contributors and others
-  http://www.digital-genes.com/~yatsu/macuim/
-
-  All rights reserved.
+  Copyright (c) 2003-2005 MacUIM contributors, All rights reserved.
 
   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions
@@ -78,8 +75,16 @@
 #define DEBUG_MODETIPS  0
 #endif
 
-#ifndef DEBUG_MODE
-#define DEBUG_MODE  0
+#ifndef DEBUG_SYNCMODE
+#define DEBUG_SYNCMODE  0
+#endif
+
+#ifndef DEBUG_INPUTMODE
+#define DEBUG_INPUTMODE  0
+#endif
+
+#ifndef DEBUG_HILITERANGE
+#define DEBUG_HILITERANGE  0
 #endif
 
 #ifndef DEBUG_SESSION
@@ -94,8 +99,12 @@
 #define DEBUG_NOTIFY  0
 #endif
 
+#ifndef DEBUG_CUSTOM
+#define DEBUG_CUSTOM  0
+#endif
+
 #ifndef DEBUG_UIM
-#define DEBUG_UIM  0
+#define DEBUG_UIM  1
 #endif
 
 #ifndef DEBUG_PRIME
