@@ -1,9 +1,6 @@
 /* -*- mode:c; coding:utf-8; tab-width:8; c-basic-offset:2; indent-tabs-mode:nil -*- */
 /*
-  Copyright (c) 2003-2004 MacUIM contributors and others
-  http://www.digital-genes.com/~yatsu/macuim/
-
-  All rights reserved.
+  Copyright (c) 2003-2005 MacUIM contributors, All rights reserved.
 
   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions
@@ -77,11 +74,15 @@ enum
   kHideSendEventPaletteMenuItemString = 4
 };
 
+Boolean gMacUIMDisable = FALSE;
+
 MUIMSessionHandle gActiveSession;
 
 int gNumSession = 0;
 MUIMSessionHandle *gSessionList = NULL;
 
+//static EventTime gDeactivateTime = 0.0;
+
 #ifdef NEW_HELPER
 Boolean gUimHelperConnected;
 #else
@@ -96,6 +97,8 @@ CFSocketContext gSockContext;
 SInt32 gMode = -1;
 #endif
 
+Boolean gActiveProp = TRUE;
+
 CFStringRef gIMName = NULL;
 Boolean gEnableModeTips = true;
 Boolean gCandVertical = true;
@@ -103,7 +106,7 @@ int gCandTransparency = 0;
 CFStringRef gCandFont = NULL;
 float gCandFontSize = 0.0;
 
-static MenuRef gPencilMenu;
+MenuRef gPencilMenu;
 
 static CFStringRef gShowKeyboardPaletteMenuItemString;
 static CFStringRef gHideKeyboardPaletteMenuItemString;
@@ -134,6 +137,7 @@ ConvertKeyVal(int inKey, int inMod);
 static int
 ConvertModifier(int inMod);
 
+#pragma mark -
 
 ComponentResult
 MUIMInitialize(ComponentInstance inComponentInstance,
@@ -180,6 +184,7 @@ MUIMInitialize(ComponentInstance inComponentInstance,
 
   if (result == noErr)
     result = NewIconSuite(&menuIconSuite);
+
   if (result == noErr) {
     iconData = GetResource('ics8', kICNx_Pencil);
     if (iconData == nil)
@@ -241,29 +246,24 @@ MUIMInitialize(ComponentInstance inComponentInstance,
   if (refNum != -1)
     CloseComponentResFile(refNum);
 
-  /*
-  if (result == noErr)
-    result = MUIMInitializeMessageReceiving();
-  */
-
 #if DEBUG_SESSION
-  DEBUG_PRINT("MUIMInitialize()\n");
+  DEBUG_PRINT("MUIMInitialize() uid=%ld euid=%ld\n",
+              getuid(), geteuid());
 #endif
 
-  if (CFPreferencesGetAppBooleanValue(CFSTR(kPrefHelperIM),
-                                      CFSTR(kAppID), &dummy))
-    gCandVertical = true;
-  else
-    gCandVertical = false;
-  
-  gCandTransparency =
-    CFPreferencesGetAppIntegerValue(CFSTR(kPrefCandTransparency),
-                                    CFSTR(kAppID), &dummy);
-  
-  // candidate font
-  {
+  if (result == noErr) {
     CFPropertyListRef propVal;
-
+    
+    if (CFPreferencesGetAppBooleanValue(CFSTR(kPrefHelperIM),
+                                        CFSTR(kAppID), &dummy))
+      gCandVertical = true;
+    else
+      gCandVertical = false;
+  
+    gCandTransparency =
+      CFPreferencesGetAppIntegerValue(CFSTR(kPrefCandTransparency),
+                                      CFSTR(kAppID), &dummy);
+  
     propVal = CFPreferencesCopyAppValue(CFSTR(kPrefCandFont), CFSTR(kAppID));
     if (propVal && CFGetTypeID(propVal) == CFStringGetTypeID()) {
       gCandFont = (CFStringRef) propVal;
@@ -273,17 +273,33 @@ MUIMInitialize(ComponentInstance inComponentInstance,
     propVal = CFPreferencesCopyAppValue(CFSTR(kPrefCandFontSize), CFSTR(kAppID));
     if (propVal && CFGetTypeID(propVal) == CFNumberGetTypeID())
       CFNumberGetValue((CFNumberRef) propVal, kCFNumberFloatType, &gCandFontSize);
+  
+    if (CFPreferencesGetAppBooleanValue(CFSTR(kPrefModeTips),
+                                        CFSTR(kAppID), &dummy))
+      gEnableModeTips = true;
+    else
+      gEnableModeTips = false;
   }
   
-  if (CFPreferencesGetAppBooleanValue(CFSTR(kPrefModeTips),
-                                      CFSTR(kAppID), &dummy))
-    gEnableModeTips = true;
-  else
-    gEnableModeTips = false;
+#ifdef NEW_HELPER
+  gUimHelperConnected = FALSE;
+#else
+  gUimFD = -1;
+#endif
   
-  {
+  gUimSock = NULL;
+  gUimRun = NULL;
+  
+  if (getuid() == 0 && getuid() != geteuid()) {
+    gMacUIMDisable = TRUE;
+    //result = resNotFound;
+    return result;
+  }
+  else {
     char *home = getenv("HOME");
-    if (home == NULL) {
+    if (!home) {
+      DEBUG_PRINT("MUIMInitialize() $HOME is NULL\n");
+#if 0
       struct passwd *pw = NULL;
       uid_t uid = getuid();
       pw = getpwuid(uid);
@@ -291,12 +307,17 @@ MUIMInitialize(ComponentInstance inComponentInstance,
         setenv("HOME", "/", 0);
       else
         setenv("HOME", pw->pw_dir, 0);
+#else
+      gMacUIMDisable = TRUE;
+      //result = resNotFound;
+      return result;
+#endif
     }
   }
 
 #if DEBUG_UIM
   // Uim debug
-  {
+  if (result == noErr) {
     char num[16];
     sprintf(num, "%d", DEBUG_UIM);
     setenv("LIBUIM_VERBOSE", num, 1);
@@ -305,23 +326,14 @@ MUIMInitialize(ComponentInstance inComponentInstance,
 
 #if DEBUG_PRIME
   // PRIME debug
-  setenv("PRIME_DEBUG", "1", 1);
+  if (result == noErr) {
+    setenv("PRIME_DEBUG", "1", 1);
+  }
 #endif
 
-  uim_init();
+  if (result == noErr) {
+    uim_init();
 
-#ifdef NEW_HELPER
-  gUimHelperConnected = FALSE;
-#else
-  gUimFD = -1;
-#endif
-
-  gUimSock = NULL;
-  gUimRun = NULL;
-
-  //CreateUIMContext(NULL);
-
-  {
     CFNotificationCenterRef center =
       CFNotificationCenterGetDistributedCenter();
 
@@ -345,30 +357,27 @@ MUIMTerminate(ComponentInstance inComponentInstance)
 #if DEBUG_SESSION
     DEBUG_PRINT("MUIMTerminate()\n");
 #endif
-
-#if ENABLE_HELPER
-    UIMHelperClose();
-#endif
-
-    uim_quit();
-
     gActiveSession = NULL;
     gPencilMenu = NULL;
+    
+    if (gMacUIMDisable)
+      return;
+    
+    UIMHelperClose();
+    uim_quit();
 }
 
-#pragma mark -
-
 ComponentResult
 MUIMSessionOpen(ComponentInstance inComponentInstance,
                 MUIMSessionHandle *outSessionHandle)
 {
   ComponentResult result = noErr;
-
+  
   if (*outSessionHandle == nil) {
     *outSessionHandle =
       (MUIMSessionHandle) NewHandle(sizeof(MUIMSessionRecord));
 #if DEBUG_SESSION
-    //DEBUG_PRINT("MUIMSessionOpen() NewHandle=%p\n", *outSessionHandle);
+    DEBUG_PRINT("MUIMSessionOpen() NewHandle=%p\n", *outSessionHandle);
 #endif
   }
 
@@ -395,19 +404,26 @@ MUIMSessionOpen(ComponentInstance inComponentInstance,
     (**outSessionHandle)->fOldPreedit = NULL;
     (**outSessionHandle)->fOldPreeditLen = 0;
 
-    //(**outSessionHandle)->fMode = -1;
-    //(**outSessionHandle)->fModeLock = false;
+    (**outSessionHandle)->fModeTipsBlock = TRUE;
 
     (**outSessionHandle)->fBundleRef = NULL;
     (**outSessionHandle)->fWindowOpened = false;
+    
+    (**outSessionHandle)->fLastMode = NULL;
 
+    (**outSessionHandle)->fDeactivateTime = -1.0;
+    
     (**outSessionHandle)->fUC = NULL;
-    CreateUIMContext(*outSessionHandle, NULL);
+    if (!gMacUIMDisable)
+      CreateUIMContext(*outSessionHandle, NULL);
   }
   else
     result = memFullErr;
 
-  InitCandidateWindow((*outSessionHandle));
+  if (result == noErr) {
+    if (!gMacUIMDisable)
+      InitCandidateWindow((*outSessionHandle));
+  }
 
   return result;
 }
@@ -421,7 +437,7 @@ MUIMSessionClose(MUIMSessionHandle inSessionHandle)
   DEBUG_PRINT("MUIMSessionClose() inSessionHandle=%p\n",
               inSessionHandle);
 #endif
-
+  
   for (i = 0; i < gNumSession; i++) {
     if (gSessionList[i] == inSessionHandle) {
       gSessionList[i] = NULL;
@@ -441,6 +457,7 @@ MUIMSessionClose(MUIMSessionHandle inSessionHandle)
     (*inSessionHandle)->fSegments = NULL;
     (*inSessionHandle)->fSegmentCount = 0;
 
+    uim_release_context((*inSessionHandle)->fUC);
     DisposeHandle((Handle) inSessionHandle);
   }
 }
@@ -454,27 +471,39 @@ MUIMSessionActivate(MUIMSessionHandle inSessionHandle)
   SetScriptVariable(smJapanese, smScriptKeys, keyboardID);
 
   gActiveSession = inSessionHandle;
+  
+  (*inSessionHandle)->fModeTipsBlock = FALSE;
 
 #if DEBUG_SESSION
-  DEBUG_PRINT("MUIMSessionActivate() gActiveSession=%p\n",
-              gActiveSession);
+  DEBUG_PRINT("MUIMSessionActivate() gActiveSession=%p (%p) time=%f deactivate=%f\n",
+              gActiveSession, &gActiveSession, GetCurrentEventTime(),
+              (*inSessionHandle)->fDeactivateTime);
 #endif
+  
+  if (gMacUIMDisable)
+    return noErr;
 
+  //DEBUG_PRINT("activate start\n");
+  
   if ((*inSessionHandle)->fWindowOpened)
     LayoutCandidate(inSessionHandle);
   
 #ifdef SYNC_MODE
-#if DEBUG_HELPER
-  DEBUG_PRINT("MUIMSessionActivate() gActiveSession=%p mode=%d gMode=%d fSegmentCount=%lu\n",
+#if DEBUG_SYNCMODE
+  DEBUG_PRINT("MUIMSessionActivate() gActiveSession=%p mode=%d gMode=%d\n",
               gActiveSession,
-              uim_get_current_mode((*inSessionHandle)->fUC), gMode,
-              (*inSessionHandle)->fSegmentCount);
+              uim_get_current_mode((*inSessionHandle)->fUC), gMode);
 #endif
 
   UIMCheckHelper();
   
+  uim_helper_client_focus_in((*inSessionHandle)->fUC);
+  
+  gActiveProp = TRUE;
+  
   if (gMode >= 0) {
     if (gMode != uim_get_current_mode((*inSessionHandle)->fUC)) {
+      // UIM mode was changed
 #if DEBUG_HELPER
       DEBUG_PRINT("MUIMSessionActivate() set mode %d\n", gMode);
 #endif
@@ -483,9 +512,22 @@ MUIMSessionActivate(MUIMSessionHandle inSessionHandle)
 #if DEBUG_HELPER
     DEBUG_PRINT("MUIMSessionActivate() label update\n");
 #endif
-    uim_prop_label_update((*inSessionHandle)->fUC);
+    if ((*inSessionHandle)->fDeactivateTime != 0.0) {
+      if ((*inSessionHandle)->fDeactivateTime + 0.05 > GetCurrentEventTime()) {
+        uim_prop_label_update((*inSessionHandle)->fUC);
+      }
+      else {
+        (*inSessionHandle)->fModeTipsBlock = TRUE;
+        uim_prop_label_update((*inSessionHandle)->fUC);
+        (*inSessionHandle)->fModeTipsBlock = FALSE;
+      }
+      (*inSessionHandle)->fDeactivateTime = 0.0;
+    }
+    else
+      uim_prop_label_update((*inSessionHandle)->fUC);
   }
   else { // gMode == -1
+    // first activation
 #if DEBUG_HELPER
     DEBUG_PRINT("MUIMSessionActivate() list update\n");
 #endif
@@ -494,14 +536,12 @@ MUIMSessionActivate(MUIMSessionHandle inSessionHandle)
 #endif // SYNC_MODE
 
 #ifndef SYNC_MODE
-#if ENABLE_HELPER
   UIMCheckHelper();
+  uim_helper_client_focus_in((*inSessionHandle)->fUC);
   uim_prop_list_update((*inSessionHandle)->fUC);
   uim_prop_label_update((*inSessionHandle)->fUC);
-  //uim_helper_client_focus_in((*inSessionHandle)->fUC);
-#endif // ENABLE_HELPER
 #endif // !SYNC_MODE
-
+  
   return result;
 }
 
@@ -509,16 +549,28 @@ ComponentResult
 MUIMSessionDeactivate(MUIMSessionHandle inSessionHandle)
 {
 #if DEBUG_SESSION
-  DEBUG_PRINT("MUIMSessionDeactivate() gActiveSession=%p\n",
-              gActiveSession);
+  DEBUG_PRINT("MUIMSessionDeactivate() gActiveSession=%p time=%f\n",
+              gActiveSession, GetCurrentEventTime());
 #endif
+  
+  if (gMacUIMDisable)
+    return noErr;
 
+  (*inSessionHandle)->fDeactivateTime = GetCurrentEventTime();
+  
   MUIMHideWindow(inSessionHandle);
 
-#if ENABLE_HELPER
-  //uim_helper_client_focus_out((*inSessionHandle)->fUC);
-#endif
+  uim_helper_client_focus_out((*inSessionHandle)->fUC);
+  
+  (*inSessionHandle)->fModeTipsBlock = TRUE;
 
+  /*
+  if ((*inSessionHandle)->fLastMode) {
+    CFRelease((*inSessionHandle)->fLastMode);
+    (*inSessionHandle)->fLastMode = NULL;
+  }
+  */
+  
   return noErr;
 }
 
@@ -530,6 +582,9 @@ MUIMSessionEvent(MUIMSessionHandle inSessionHandle, EventRef inEventRef)
   UInt32 eventKind;
 
   handled = FALSE;
+  
+  if (gMacUIMDisable)
+    return handled;
 
   eventClass = GetEventClass(inEventRef);
   eventKind = GetEventKind(inEventRef);
@@ -551,6 +606,17 @@ MUIMSessionEvent(MUIMSessionHandle inSessionHandle, EventRef inEventRef)
   //   kEventRawKeyUp - A key was released
   //   kEventRawKeyModifiersChanged - The keyboard modifiers (bucky bits) have changed
 
+  /*
+  if (eventKind == kEventRawKeyDown)
+    DEBUG_PRINT("key down\n");
+  else if (eventKind == kEventRawKeyRepeat)
+    DEBUG_PRINT("key repeat\n");
+  else if (eventKind == kEventRawKeyUp)
+    DEBUG_PRINT("key up\n");
+  else if (eventKind == kEventRawKeyModifiersChanged)
+    DEBUG_PRINT("mod changed\n");
+  */
+  
   if (eventClass == kEventClassKeyboard &&
       (eventKind == kEventRawKeyDown ||
        eventKind == kEventRawKeyRepeat)) {
@@ -589,35 +655,107 @@ MUIMSessionFix(MUIMSessionHandle inSessionHandle)
   DEBUG_PRINT("MUIMSessionFix() fFixLen=%lu\n", (*inSessionHandle)->fFixLen);
 #endif
 
+  if (gMacUIMDisable)
+    return noErr;
+  
   if ((*inSessionHandle)->fFixLen == 0) {
     // commit preedit string (single context)
     GetPreeditString(inSessionHandle, &str, &len, &cursorPos);
     if (len > 0) {
-#if 0
-      (*inSessionHandle)->fFixBuffer = str;
-      (*inSessionHandle)->fFixLen = len;
-
-      MUIMUpdateActiveInputArea(inSessionHandle, TRUE);
-
-      free((*inSessionHandle)->fFixBuffer);
-      (*inSessionHandle)->fFixBuffer = NULL;
-      (*inSessionHandle)->fFixLen = 0;
-
-      uim_reset_context((*inSessionHandle)->fUC);
-#else
       uim_press_key((*inSessionHandle)->fUC, 'j', UMod_Control);
       uim_release_key((*inSessionHandle)->fUC, 'j', UMod_Control);
-#endif
     }
-
-#if ENABLE_HELPER
     MUIMHideWindow(inSessionHandle);
-#endif
   }
 
   return noErr;
 }
 
+ComponentResult
+MUIMSetInputMode(MUIMSessionHandle inSessionHandle, CFStringRef inInputMode)
+{
+  ComponentResult result;
+  
+  if (!inSessionHandle || !(*inSessionHandle) ||
+      !(*inSessionHandle)->fUC || !inInputMode)
+    return tsmInputModeChangeFailedErr;
+  
+#if DEBUG_INPUTMODE
+  DEBUG_PRINT("MUIMSetInputMode() inInputModer='%s'\n",
+              CFStringGetCStringPtr(inInputMode, kCFStringEncodingMacRoman));
+#endif
+  
+  if (gMacUIMDisable)
+    return tsmInputModeChangeFailedErr;
+  
+  TSMSelectInputMode((Component) (*inSessionHandle)->fComponentInstance,
+                     inInputMode);
+  
+  if (!(*inSessionHandle)->fLastMode ||
+      CFStringCompare(inInputMode, (*inSessionHandle)->fLastMode, 0)) {
+    
+    //DEBUG_PRINT("MUIMSetInputMode() differenct mode\n");
+  
+    if (!CFStringCompare(inInputMode,
+                         kTextServiceInputModeRoman, 0) ||
+        !CFStringCompare(inInputMode,
+                         kTextServiceInputModePassword, 0)) {
+#if 0
+      UInt32 i;
+      for (i = 0; i < gNumSession; i++) {
+        BlockUpdatePreedit();
+        uim_press_key((*gSessionList[i])->fUC, UKey_Private1, 0);
+        uim_release_key((*gSessionList[i])->fUC, UKey_Private1, 0);
+        UnblockUpdatePreedit();
+      }
+#else
+      BlockUpdatePreedit();
+      uim_press_key((*inSessionHandle)->fUC, UKey_Private1, 0);
+      uim_release_key((*inSessionHandle)->fUC, UKey_Private1, 0);
+      UnblockUpdatePreedit();
+#endif
+    }
+    else if (!CFStringCompare(inInputMode,
+                              kTextServiceInputModeJapanese, 0) ||
+             !CFStringCompare(inInputMode,
+                              kTextServiceInputModeJapaneseHiragana, 0) ||
+             !CFStringCompare(inInputMode,
+                              kTextServiceInputModeJapaneseKatakana, 0) ||
+             !CFStringCompare(inInputMode,
+                              kTextServiceInputModeJapaneseHalfWidthKana, 0) ||
+             !CFStringCompare(inInputMode,
+                              kTextServiceInputModeJapaneseFullWidthRoman, 0) ||
+             !CFStringCompare(inInputMode,
+                              kTextServiceInputModeJapaneseFirstName, 0) ||
+             !CFStringCompare(inInputMode,
+                              kTextServiceInputModeJapaneseLastName, 0) ||
+             !CFStringCompare(inInputMode,
+                              kTextServiceInputModeJapanesePlaceName, 0)) {
+#if 0
+      UInt32 i;
+      for (i = 0; i < gNumSession; i++) {
+        BlockUpdatePreedit();
+        uim_press_key((*gSessionList[i])->fUC, UKey_Private2, 0);
+        uim_release_key((*gSessionList[i])->fUC, UKey_Private2, 0);
+        UnblockUpdatePreedit();
+      }
+#else
+      BlockUpdatePreedit();
+      uim_press_key((*inSessionHandle)->fUC, UKey_Private2, 0);
+      uim_release_key((*inSessionHandle)->fUC, UKey_Private2, 0);
+      UnblockUpdatePreedit();
+#endif
+    }
+
+    if ((*inSessionHandle)->fLastMode)
+      CFRelease((*inSessionHandle)->fLastMode);
+    (*inSessionHandle)->fLastMode = inInputMode;
+    CFRetain((*inSessionHandle)->fLastMode);
+  }
+  
+  return result;
+}
+
 ComponentResult
 MUIMHideWindow(MUIMSessionHandle inSessionHandle)
 {
@@ -630,8 +768,6 @@ MUIMHideWindow(MUIMSessionHandle inSessionHandle)
   return noErr;
 }
 
-#pragma mark -
-
 MUIMSessionHandle
 MUIMGetActiveSession(void)
 {
@@ -666,10 +802,10 @@ MUIMHandleInput(MUIMSessionHandle inSessionHandle, UInt32 inKeycode,
       (inCharCode >= '0' && inCharCode <= '9')) {
     UInt32 row = inCharCode == '0' ? 9 : inCharCode - '1';
 #if DEBUG_KEYEVENT
-    DEBUG_PRINT("MUIMHandleInput() select candidate %lu index=%ld begin=%ld end=%ld\n",
+    DEBUG_PRINT("MUIMHandleInput() select candidate %lu index=%ld begin=%ld end=%ld row=%lu\n",
                 row, (*inSessionHandle)->fCandidateIndex,
                 (*inSessionHandle)->fLayoutBegin,
-                (*inSessionHandle)->fNRCandidates);
+                (*inSessionHandle)->fNRCandidates, row);
 #endif
     if ((*inSessionHandle)->fLayoutBegin + row <
         (*inSessionHandle)->fNRCandidates)
@@ -686,7 +822,7 @@ MUIMHandleInput(MUIMSessionHandle inSessionHandle, UInt32 inKeycode,
   }
   if (key == UKey_Private1 || key == UKey_Private2)
     isScriptKey = TRUE;
-  
+    
   /* Then convert normal keys */
   if (key == 0) {
     key = ConvertKeyVal(inCharCode, inModifiers);
@@ -710,7 +846,6 @@ MUIMHandleInput(MUIMSessionHandle inSessionHandle, UInt32 inKeycode,
 #endif
 
   rv = uim_press_key((*inSessionHandle)->fUC, key, mod);
-
   uim_release_key((*inSessionHandle)->fUC, key, mod);
 
   if (!rv || isScriptKey)
@@ -720,7 +855,7 @@ MUIMHandleInput(MUIMSessionHandle inSessionHandle, UInt32 inKeycode,
   DEBUG_PRINT("MUIMHandleInput() uim_press_key handled=%s\n",
               handled ? "true" : "false");
 #endif
-
+  
   return handled;
 }
 
@@ -760,8 +895,6 @@ MUIMUpdateShowHideSendEventPaletteMenuItem(Boolean inIsHidden)
                                 gHideSendEventPaletteMenuItemString);
 }
 
-#pragma mark -
-
 static pascal OSStatus
 MUIMPencilMenuEventHandler(EventHandlerCallRef inEventHandlerCallRef,
                            EventRef inEventRef, void *inUserData)
@@ -801,6 +934,8 @@ MUIMPencilMenuEventHandler(EventHandlerCallRef inEventHandlerCallRef,
   return result;
 }
 
+#pragma mark -
+
 static void
 CreateUIMContext(MUIMSessionHandle inSessionHandle, CFStringRef inIMName)
 {
@@ -839,7 +974,6 @@ CreateUIMContext(MUIMSessionHandle inSessionHandle, CFStringRef inIMName)
                      UIMPreeditPushback,
                      UIMPreeditUpdate);
 
-#if ENABLE_HELPER
   UIMCheckHelper();
   uim_set_prop_list_update_cb((*inSessionHandle)->fUC, UIMUpdatePropList);
   uim_set_prop_label_update_cb((*inSessionHandle)->fUC, UIMUpdatePropLabel);
@@ -847,7 +981,6 @@ CreateUIMContext(MUIMSessionHandle inSessionHandle, CFStringRef inIMName)
   uim_prop_list_update((*inSessionHandle)->fUC);
   uim_prop_label_update((*inSessionHandle)->fUC);
 #endif // SYNC_MODE
-#endif // ENABLE_HELPER
 
   uim_set_candidate_selector_cb((*inSessionHandle)->fUC,
                                 UIMCandAcivate,
@@ -886,6 +1019,26 @@ ReleaseAllUIMContext()
   }
 }
 
+#pragma mark -
+
+void
+CandClicked(UInt32 inRow)
+{
+  UInt32 idx = 0;
+  
+  idx = (*gActiveSession)->fDisplayLimit * (*gActiveSession)->fPageIndex + inRow;
+  
+#if DEBUG_CANDIDATES
+  DEBUG_PRINT("CandClicked() inRow=%lu fPageIndex=%ld idx=%lu\n",
+              inRow, (*gActiveSession)->fPageIndex, idx);
+#endif
+  
+  uim_set_candidate_index((*gActiveSession)->fUC, idx);
+  
+  uim_press_key((*gActiveSession)->fUC, 'j', UMod_Control);
+  uim_release_key((*gActiveSession)->fUC, 'j', UMod_Control);
+}
+
 static void
 NotificationCallback(CFNotificationCenterRef inCenter, void *inObserver, 
                      CFStringRef inName, const void *inObject, 
@@ -949,6 +1102,7 @@ NotificationCallback(CFNotificationCenterRef inCenter, void *inObserver,
     CreateAllUIMContext(im);
 
 #ifdef SYNC_MODE
+    DEBUG_PRINT("NotificationCallback() gMode=%d -> -1\n", gMode);
     gMode = -1;
     for (i = 0; i < gNumSession; i++) {
       uim_prop_list_update((*gSessionList[i])->fUC);
@@ -961,6 +1115,8 @@ NotificationCallback(CFNotificationCenterRef inCenter, void *inObserver,
   }
 }
 
+#pragma mark -
+
 /*
  * key codes are defined in
  * /System/Library/Frameworks/Carbon.framework/Frameworks/HIToolbox.framework/Headers/Events.h
