@@ -1,9 +1,6 @@
 /* -*- mode:c; coding:utf-8; tab-width:8; c-basic-offset:2; indent-tabs-mode:nil -*- */
 /*
-  Copyright (c) 2003-2004 MacUIM contributors and others
-  http://www.digital-genes.com/~yatsu/macuim/
-
-  All rights reserved.
+  Copyright (c) 2003-2005 MacUIM contributors, All rights reserved.
 
   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions
@@ -47,14 +44,15 @@ extern MUIMSessionHandle gActiveSession;
 long gInstanceRefCount = 0;
 MenuRef gTextServiceMenu = nil;
 
-//SInt32 gProp = kTextServiceInputModePropertyTag;
+static ComponentResult
+CallMUIMFunction(ComponentParameters *inParams, ProcPtr inProcPtr,
+                 SInt32 inProcInfo);
 
-static ComponentResult CallMUIMFunction(ComponentParameters * inParams,
-                                        ProcPtr inProcPtr, SInt32 inProcInfo);
-static ComponentResult CallMUIMFunctionWithStorage(Handle inStorage,
-                                                   ComponentParameters *
-                                                   inParams, ProcPtr inProcPtr,
-                                                   SInt32 inProcInfo);
+static ComponentResult
+CallMUIMFunctionWithStorage(Handle inStorage, ComponentParameters *inParams,
+                            ProcPtr inProcPtr, SInt32 inProcInfo);
+
+#pragma mark -
 
 /**
  * This routine is the main entry point for our text service component.
@@ -89,14 +87,13 @@ MUIMComponentDispatch(ComponentParameters * inParams, Handle inSessionHandle)
     break;
 
   case kComponentCanDoSelect:
-    result =
-      CallMUIMFunction(inParams, (ProcPtr) MUIMCanDo, uppCanDoProcInfo);
+    result = CallMUIMFunction(inParams, (ProcPtr) MUIMCanDo,
+                              uppCanDoProcInfo);
     break;
 
   case kComponentVersionSelect:
-    result =
-      CallMUIMFunction(inParams, (ProcPtr) MUIMGetVersion,
-                       uppGetVersionProcInfo);
+    result = CallMUIMFunction(inParams, (ProcPtr) MUIMGetVersion,
+                              uppGetVersionProcInfo);
     break;
 
   case kCMGetScriptLangSupport:
@@ -148,37 +145,43 @@ MUIMComponentDispatch(ComponentParameters * inParams, Handle inSessionHandle)
     break;
 
   case kCMHidePaletteWindows:
-    //DEBUG_PRINT("MUIMComponentDispatch() kCMHidePaletteWindows\n");
     break;
 
-  /*
+#if 0
   case kCMGetTextServiceProperty:
-    DEBUG_PRINT("MUIMComponentDispatch() kCMGetTextServiceProperty\n");
     result = CallMUIMFunctionWithStorage(inSessionHandle, inParams,
                                          (ProcPtr) MUIMGetTextServiceProperty,
                                          uppGetTextServicePropertyProcInfo);
     break;
+#endif
 
   case kCMSetTextServiceProperty:
-    DEBUG_PRINT("MUIMComponentDispatch() kCMSettTextServiceProperty\n");
     result = CallMUIMFunctionWithStorage(inSessionHandle, inParams,
                                          (ProcPtr) MUIMSetTextServiceProperty,
                                          uppSetTextServicePropertyProcInfo);
     break;
 
+  /*
   case kCMUCTextServiceEvent:
     DEBUG_PRINT("MUIMComponentDispatch() kCMUCTextServiceEvent¥n");
     break;
-  */
+  case kCMCopyTextServiceInputModeList:
+  */    
 
   default:
     result = badComponentSelector;
     break;
   }
-
+  
+#if DEBUG_COMPONENT
+  DEBUG_PRINT("MUIMComponentDispatch() result=%d\n", result);
+#endif
+  
   return result;
 }
 
+#pragma mark -
+
 /**
  * This routine is called directly via OpenComponent, or indirectly via
  * NewTSMDocument.
@@ -274,9 +277,6 @@ MUIMCanDo(SInt16 inSelector)
   case kComponentCloseSelect:
   case kComponentCanDoSelect:
   case kComponentVersionSelect:
-    result = true;
-    break;
-
   case kCMGetScriptLangSupport:
   case kCMInitiateTextService:
   case kCMTerminateTextService:
@@ -286,11 +286,9 @@ MUIMCanDo(SInt16 inSelector)
   case kCMGetTextServiceMenu:
   case kCMFixTextService:
   case kCMHidePaletteWindows:
-    /*
-  case kCMGetTextServiceProperty: // XXX
-  case kCMSetTextServiceProperty: // XXX
-  case kCMUCTextServiceEvent:     // XXX
-    */
+  //case kCMGetTextServiceProperty:
+  case kCMSetTextServiceProperty:
+  //case kCMUCTextServiceEvent:
     result = true;
     break;
 
@@ -333,7 +331,7 @@ MUIMGetScriptLangSupport(Handle inSessionHandle,
 #pragma unused (inSessionHandle)
 
 #if DEBUG_COMPONENT
-  DEBUG_PRINT("MUIMGetScriptLangSupport()^n");
+  DEBUG_PRINT("MUIMGetScriptLangSupport()\n");
 #endif
 
   OSStatus result;
@@ -498,6 +496,7 @@ MUIMGetTextServiceMenu(Handle inSessionHandle, MenuHandle * outMenuHandle)
 #endif
 
   *outMenuHandle = gTextServiceMenu;
+  
   return noErr;
 }
 
@@ -520,25 +519,46 @@ MUIMFixTextService(Handle inSessionHandle)
   return MUIMSessionFix((MUIMSessionHandle) inSessionHandle);
 }
 
-/*
+#if 0
 pascal ComponentResult
 MUIMGetTextServiceProperty(Handle inSessionHandle, OSType inPropSelector,
-                           SInt32 *inProp)
+                           SInt32 *outProp)
 {
-  *inProp = gProp;
-
+#if DEBUG_INPUTMODE
+  DEBUG_PRINT("MUIMGetTextServiceProperty %ul %ul\n",
+              inPropSelector, kTextServiceInputModePropertyTag);
+#endif
+  
   return noErr;
 }
+#endif
 
 pascal ComponentResult
 MUIMSetTextServiceProperty(Handle inSessionHandle, OSType inPropSelector,
                            SInt32 inProp)
 {
-  gProp = inProp;
+  ComponentResult result;
+  
+  switch (inPropSelector) {
+  //case kTextServiceJaTypingMethodPropertyTag:
+  case kTextServiceInputModePropertyTag:
+  case kIMJaTypingMethodProperty:
+  case kIMJaTypingMethodRoman:
+  case kIMJaTypingMethodKana:
+    DEBUG_PRINT("MUIMSetTextServiceProperty() inPropSelector=%ld\n",
+                inPropSelector);
+    result = noErr;
+    break;
 
-  return noErr;
+  default:
+    result = MUIMSetInputMode((MUIMSessionHandle) inSessionHandle,
+                              (CFStringRef) inPropSelector);
+  }
+
+  return result;
 }
-*/
+
+#pragma mark -
 
 /**
  * Glue code to create a Universal Procedure Pointer for our internal dispatch
@@ -551,16 +571,16 @@ MUIMSetTextServiceProperty(Handle inSessionHandle, OSType inPropSelector,
  * @return ComponentResult  A toolbox error code.
  */
 static ComponentResult
-CallMUIMFunction(ComponentParameters * inParams, ProcPtr inProcPtr,
+CallMUIMFunction(ComponentParameters *inParams, ProcPtr inProcPtr,
                  SInt32 inProcInfo)
 {
-  ComponentResult result;
+  ComponentResult result = noErr;
   ComponentFunctionUPP componentFunctionUPP;
 
-  result = noErr;
   componentFunctionUPP = NewComponentFunctionUPP(inProcPtr, inProcInfo);
   result = CallComponentFunction(inParams, componentFunctionUPP);
   DisposeComponentFunctionUPP(componentFunctionUPP);
+  
   return result;
 }
 
@@ -577,18 +597,17 @@ CallMUIMFunction(ComponentParameters * inParams, ProcPtr inProcPtr,
  * @return ComponentResult  A toolbox error code.
  */
 static ComponentResult
-CallMUIMFunctionWithStorage(Handle inStorage,
-                            ComponentParameters * inParams,
+CallMUIMFunctionWithStorage(Handle inStorage, ComponentParameters *inParams,
                             ProcPtr inProcPtr, SInt32 inProcInfo)
 {
-  ComponentResult result;
+  ComponentResult result = noErr;
   ComponentFunctionUPP componentFunctionUPP;
 
-  result = noErr;
   componentFunctionUPP = NewComponentFunctionUPP(inProcPtr, inProcInfo);
   result =
     CallComponentFunctionWithStorage(inStorage, inParams,
                                      componentFunctionUPP);
   DisposeComponentFunctionUPP(componentFunctionUPP);
+  
   return result;
 }
