@@ -1,9 +1,6 @@
 /* -*- mode:objc; coding:utf-8; tab-width:8; c-basic-offset:2; indent-tabs-mode:nil -*- */
 /*
-  Copyright (c) 2003-2004 MacUIM contributors and others
-  http://www.digital-genes.com/~yatsu/macuim/
-
-  All rights reserved.
+  Copyright (c) 2003-2005 MacUIM contributors, All rights reserved.
 
   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions
@@ -304,9 +301,11 @@ convertHelperString(char *str);
     NSString *mode;
     NSMenuItem *menuItem;
     mode = [modes objectAtIndex:i];
-    menuItem = (NSMenuItem *) [[menu addItemWithTitle:mode
-                                     action:@selector(modeSelect:) keyEquivalent:@""]
-                                setTarget:self];
+    menuItem = (NSMenuItem *)
+      [menu addItemWithTitle:mode
+                      action:@selector(modeSelect:)
+                keyEquivalent:@""];
+    [menuItem setTarget:self];
     [menuItems addObject:menuItem];
   }
 
@@ -317,10 +316,10 @@ convertHelperString(char *str);
 
   for (i = 0; i < [imNames count]; i++) {
     NSMenuItem *menuItem;
-    menuItem = (NSMenuItem *) [[menu addItemWithTitle:[NSString stringWithFormat:@"    %@",
-                                                                [imNames objectAtIndex:i]]
-                                     action:@selector(imSelect:) keyEquivalent:@""]
-                                setTarget:self];
+    menuItem = (NSMenuItem *)
+      [menu addItemWithTitle:[NSString stringWithFormat:@"    %@", [imNames objectAtIndex:i]]
+                      action:@selector(imSelect:) keyEquivalent:@""];
+    [menuItem setTarget:self];
     if (imName && [imName compare:[imNames objectAtIndex:i]] == NSOrderedSame)
       [menuItem setState:NSOnState];
     [imItems addObject:menuItem];
