@@ -1,9 +1,6 @@
 /* -*- mode:objc; coding:utf-8; tab-width:8; c-basic-offset:2; indent-tabs-mode:nil -*- */
 /*
-  Copyright (c) 2003-2004 MacUIM contributors and others
-  http://www.digital-genes.com/~yatsu/macuim/
-
-  All rights reserved.
+  Copyright (c) 2003-2005 MacUIM contributors, All rights reserved.
 
   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions
@@ -31,11 +28,20 @@
   SUCH DAMAGE.
 */
 
-#import "unistd.h"
+#import <unistd.h>
 #import <uim/uim.h>
+
+#import "Debug.h"
 #import "Preference.h"
 #import "MacUIMPrefPane.h"
 
+#if 0
+#import <libintl.h>
+
+#define PACKAGE    "uim"
+#define LOCALEDIR  "/Library/Frameworks/UIM.framework/Versions/Current/share/locale"
+#endif
+
 static MacUIMPrefPane *sharedPane;
 
 static void
@@ -55,8 +61,9 @@ prefChanged(CFNotificationCenterRef inCenter, void *inObserver,
  */
 - (void)dealloc
 {
+  int i;
+  
   if (numModules > 0 && imModules) {
-    int i;
     for (i = 0; i < numModules; i++) {
       free(imModules[i]->name);
       free(imModules[i]->lang);
@@ -78,8 +85,13 @@ prefChanged(CFNotificationCenterRef inCenter, void *inObserver,
 
 - (id)initWithBundle:(NSBundle *)bundle
 {
+#if 0
+  NSUserDefaults *defs;
+  NSArray *langs;
+  NSString *lang;
+#endif
   uim_context uc;
-
+  
   // Initialize the location of our preferences
   if ((self = [super initWithBundle:bundle]) != nil) {
     appID = CFSTR(kAppID);
@@ -93,14 +105,29 @@ prefChanged(CFNotificationCenterRef inCenter, void *inObserver,
 
   numModules = 0;
   imModules = NULL;
-
+  
+#if 0
+  defs = [NSUserDefaults standardUserDefaults];
+  lang = [langs objectAtIndex:0];
+  
+  if (!lang)
+    setlocale(LC_ALL, "");
+  else
+    setlocale(LC_ALL, [lang cString]);
+  
+  bindtextdomain(PACKAGE, LOCALEDIR);
+  textdomain(PACKAGE);
+  bind_textdomain_codeset(PACKAGE, "UTF-8");
+#endif
+  
+  // load IM modules
   uim_init();
   uc = uim_create_context(NULL, "UTF-8",
                           NULL, NULL, uim_iconv, NULL);
   if (uc) {
     numModules = uim_get_nr_im(uc);
     imModules = (IMModule **) malloc(sizeof(IMModule *) * numModules);
-    if (*imModules != NULL) {
+    if (imModules != NULL) {
       int i;
       for (i = 0; i < numModules; i++) {
         imModules[i] = (IMModule *) malloc(sizeof(IMModule));
@@ -135,6 +162,7 @@ prefChanged(CFNotificationCenterRef inCenter, void *inObserver,
     uim_release_context(uc);
   }
 
+  // set notification observer
   {
     CFNotificationCenterRef center =
       CFNotificationCenterGetDistributedCenter();
@@ -157,7 +185,9 @@ prefChanged(CFNotificationCenterRef inCenter, void *inObserver,
   [imButton removeAllItems];
 
   for (i = 0; i < numModules; i++) {
-    if (strlen(imModules[i]->lang) > 0)
+    //NSLog(@"mainViewDidLoad - imModules[%d]=%p", i, imModules[i]);
+    if (!imModules[i]) continue;
+    if (imModules[i]->lang && strlen(imModules[i]->lang) > 0)
       [imButton addItemWithTitle:[NSString stringWithFormat:@"%s (%s)",
                                            imModules[i]->name,
                                            imModules[i]->lang]];
@@ -171,22 +201,25 @@ prefChanged(CFNotificationCenterRef inCenter, void *inObserver,
   propVal = CFPreferencesCopyAppValue(CFSTR(kPrefIM), appID);
   if (propVal && CFGetTypeID(propVal) == CFStringGetTypeID()) {
     NSString *current = (NSString *) propVal;
+    if (!current) goto reset;
     for (i = 0; i < numModules; i++) {
       if ([current isEqualToString:[NSString stringWithCString:imModules[i]->name]]) {
         [imButton selectItemAtIndex:i];
-        break;
+        goto done;
       }
     }
   }
-  else {
-    for (i = 0; i < numModules; i++) {
-      if ([[NSString stringWithCString:kDefaultIM]
-            isEqualToString:[NSString stringWithCString:imModules[i]->name]]) {
-        [imButton selectItemAtIndex:i];
-        break;
-      }
+
+reset:
+  for (i = 0; i < numModules; i++) {
+    if ([[NSString stringWithCString:kDefaultIM]
+          isEqualToString:[NSString stringWithCString:imModules[i]->name]]) {
+      [imButton selectItemAtIndex:i];
+      break;
     }
   }
+  
+done:
   if (propVal)
     CFRelease(propVal);
 
@@ -485,15 +518,13 @@ prefChanged(CFNotificationCenterRef inCenter, void *inObserver,
 {
   int sleepCount = 0;
   NSURL *crackerUrl;
-  NSURL *helperUrl =
-    [NSURL fileURLWithPath:[[self bundle]
-                             pathForResource:@"MacUIMHelper"
-                             ofType:@"menu"
-                             inDirectory:@""]];
+  NSURL *helperUrl = [NSURL fileURLWithPath:[[self bundle]
+                              pathForResource:@"MacUIMHelper"
+                                       ofType:@"menu"
+                                  inDirectory:@""]];
 
   // Try default MenuCracker installed by user
-  CoreMenuExtraAddMenuExtra((CFURLRef) helperUrl,
-                            0, 0, 0, 0, 0);
+  CoreMenuExtraAddMenuExtra((CFURLRef) helperUrl, 0, 0, 0, 0, 0);
 
   while (![self isExtraLoaded:kHelperID]
          && (sleepCount < 2000000)) {
@@ -506,8 +537,8 @@ prefChanged(CFNotificationCenterRef inCenter, void *inObserver,
     if (![self isExtraLoaded:kMenuCrackerBundleID]) {
       // Load the cracker
       crackerUrl = [NSURL fileURLWithPath:[[self bundle] pathForResource:@"MenuCracker"
-                                                         ofType:@"menu"
-                                                         inDirectory:@""]];
+                                                                  ofType:@"menu"
+                                                             inDirectory:@""]];
       CoreMenuExtraAddMenuExtra((CFURLRef) crackerUrl,
                                 0, 0, 0, 0, 0);
       // Load the request again
