@@ -1,9 +1,6 @@
 /* -*- mode:c; coding:utf-8; tab-width:8; c-basic-offset:2; indent-tabs-mode:nil -*- */
 /*
-  Copyright (c) 2003-2004 MacUIM contributors and others
-  http://www.digital-genes.com/~yatsu/macuim/
-
-  All rights reserved.
+  Copyright (c) 2003-2005 MacUIM contributors, All rights reserved.
 
   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions
@@ -44,6 +41,8 @@ extern MUIMSessionHandle gActiveSession;
 extern int gNumSession;
 extern MUIMSessionHandle *gSessionList;
 
+extern MenuRef gPencilMenu;
+
 #ifdef SYNC_MODE
 extern SInt32 gMode;
 #endif
@@ -57,9 +56,15 @@ extern CFSocketRef gUimSock;
 extern CFRunLoopSourceRef gUimRun;
 extern CFSocketContext gSockContext;
 
+extern Boolean gActiveProp;
+
 extern Boolean gEnableModeTips;
 extern Boolean gCandVertical;
 
+static Boolean gBlockUpdatePreedit = false;
+
+#pragma mark -
+
 static void
 AddPreeditSegment(MUIMSessionHandle inHandle, int inAttr,
                   const char *inStr);
@@ -74,12 +79,16 @@ ParseHelperString(const char *str);
 static void
 ParseIMChangeString(const char *str);
 
+#pragma mark -
+
 void
 UIMCommitString(void *ptr, const char *str)
 {
-  //MUIMSessionHandle handle = (MUIMSessionHandle) ptr;
-  MUIMSessionHandle handle = gActiveSession;
+  MUIMSessionHandle handle = (MUIMSessionHandle) ptr;
+  //MUIMSessionHandle handle = gActiveSession;
   CFMutableStringRef cf_string;
+  
+  if (!handle || !(*handle)) return;
 
   cf_string = CFStringCreateMutable(NULL, 0);
   CFStringAppendCString(cf_string, str, kCFStringEncodingUTF8);
@@ -108,13 +117,15 @@ UIMCommitString(void *ptr, const char *str)
 void
 UIMPreeditClear(void *ptr)
 {
-  //MUIMSessionHandle handle = (MUIMSessionHandle) ptr;
-  MUIMSessionHandle handle = gActiveSession;
+  MUIMSessionHandle handle = (MUIMSessionHandle) ptr;
+  //MUIMSessionHandle handle = gActiveSession;
   UInt32 i;
 
 #if DEBUG_PREEDIT
   DEBUG_PRINT("UIMPreeditClear()\n");
 #endif
+  
+  if (!handle || !(*handle)) return;
 
   for (i = 0; i < (*handle)->fSegmentCount; i++)
     free((*handle)->fSegments[i].fBuffer);
@@ -127,9 +138,11 @@ UIMPreeditClear(void *ptr)
 void
 UIMPreeditPushback(void *ptr, int attr, const char *str)
 {
-  //MUIMSessionHandle handle = (MUIMSessionHandle) ptr;
-  MUIMSessionHandle handle = gActiveSession;
+  MUIMSessionHandle handle = (MUIMSessionHandle) ptr;
+  //MUIMSessionHandle handle = gActiveSession;
 
+  if (!handle || !(*handle)) return;
+  
 #if DEBUG_PREEDIT
   {
     char attrstr[50] = { ' ', '\0' };
@@ -163,14 +176,18 @@ UIMPreeditPushback(void *ptr, int attr, const char *str)
 void
 UIMPreeditUpdate(void *ptr)
 {
-  //MUIMSessionHandle handle = (MUIMSessionHandle) ptr;
-  MUIMSessionHandle handle = gActiveSession;
+  MUIMSessionHandle handle = (MUIMSessionHandle) ptr;
+  //MUIMSessionHandle handle = gActiveSession;
 
+  if (!handle || !(*handle)) return;
+  
 #if DEBUG_PREEDIT
   DEBUG_PRINT("UIMPreeditUpdate()\n");
 #endif
 
-  MUIMUpdateActiveInputArea(handle, FALSE);
+  // Avoid a crash when Firefox is launched with uim(Japanese)
+  if (!gBlockUpdatePreedit || (*handle)->fSegmentCount > 0)
+    MUIMUpdateActiveInputArea(handle, false);
 }
 
 /**
@@ -179,9 +196,11 @@ UIMPreeditUpdate(void *ptr)
 void
 UIMCandAcivate(void *inPtr, int inNR, int inLimit)
 {
-  //MUIMSessionHandle handle = (MUIMSessionHandle) inPtr;
-  MUIMSessionHandle handle = gActiveSession;
+  MUIMSessionHandle handle = (MUIMSessionHandle) inPtr;
+  //MUIMSessionHandle handl/e = gActiveSession;
 
+  if (!handle || !(*handle)) return;
+  
 #if DEBUG_CANDIDATES
   DEBUG_PRINT("UIMCandAcivate() inNR=%d inLimit=%d\n", inNR, inLimit);
 #endif
@@ -206,9 +225,11 @@ UIMCandAcivate(void *inPtr, int inNR, int inLimit)
 void
 UIMCandSelect(void *inPtr, int inIndex)
 {
-  //MUIMSessionHandle handle = (MUIMSessionHandle) inPtr;
-  MUIMSessionHandle handle = gActiveSession;
+  MUIMSessionHandle handle = (MUIMSessionHandle) inPtr;
+  //MUIMSessionHandle handle = gActiveSession;
 
+  if (!handle || !(*handle)) return;
+  
 #if DEBUG_CANDIDATES
   DEBUG_PRINT("UIMCandSelect() inIndex=%d\n", inIndex);
 #endif
@@ -225,11 +246,13 @@ UIMCandSelect(void *inPtr, int inIndex)
 void
 UIMCandShiftPage(void *inPtr, int inForward)
 {
-  //MUIMSessionHandle handle = (MUIMSessionHandle) inPtr;
-  MUIMSessionHandle handle = gActiveSession;
+  MUIMSessionHandle handle = (MUIMSessionHandle) inPtr;
+  //MUIMSessionHandle handle = gActiveSession;
+  
+  if (!handle || !(*handle)) return;
 
   if (inForward) { // next page
-    if ((*handle)->fNRCandidates <
+    if ((*handle)->fNRCandidates <=
         (*handle)->fCandidateIndex + (*handle)->fDisplayLimit) {
       // if the next page is not exist
       // move to top page and select first candidate
@@ -274,6 +297,8 @@ UIMCandDeactivate(void *inPtr)
   MUIMSessionHandle handle = (MUIMSessionHandle) inPtr;
   //MUIMSessionHandle handle = gActiveSession;
 
+  if (!handle || !(*handle)) return;
+  
 #if DEBUG_CANDIDATES
   DEBUG_PRINT("UIMCandDeactivate()\n");
 #endif
@@ -285,16 +310,18 @@ UIMCandDeactivate(void *inPtr)
 void
 UIMUpdatePropList(void *inPtr, const char *inStr)
 {
-  //MUIMSessionHandle handle = (MUIMSessionHandle) inPtr;
+  MUIMSessionHandle handle = (MUIMSessionHandle) inPtr;
   //MUIMSessionHandle handle = gActiveSession;
   char *tmp;
+  
+  if (!handle || !(*handle)) return;
 
 #if DEBUG_HELPER
   //DEBUG_PRINT("UIMUpdatePropList() handle=%p inStr='%s'\n",
   //            handle, inStr);
 #endif
 
-  //if (!gActiveSession) return;
+  if (!gActiveSession || !gPencilMenu) return;
 
   tmp = (char *) malloc(sizeof(char) *
                         (strlen(kPropListUpdate) + 1 +
@@ -326,12 +353,17 @@ UIMUpdatePropLabel(void *inPtr, const char *inStr)
   //MUIMSessionHandle handle = gActiveSession;
   char *tmp;
 
+  if (!handle || !(*handle)) return;
+
 #if DEBUG_HELPER
-  DEBUG_PRINT("UIMUpdatePropLabel() handle=%p gActiveSession=%p inStr='%s'\n",
-              handle, gActiveSession, inStr);
+  DEBUG_PRINT("UIMUpdatePropLabel() time=%f handle=%p gActiveSession=%p\n",
+              GetCurrentEventTime(),
+              handle, gActiveSession);
+  //DEBUG_PRINT("UIMUpdatePropLabel() handle=%p gActiveSession=%p inStr='%s'\n",
+  //            handle, gActiveSession, inStr);
 #endif
 
-  //if (!gActiveSession) return;
+  if (!gActiveSession || !gPencilMenu) return;
 
   tmp = (char *) malloc(sizeof(char) *
                         (strlen(kPropLabelUpdate) + 1 +
@@ -354,26 +386,25 @@ UIMUpdatePropLabel(void *inPtr, const char *inStr)
     }
     free(tmp);
   }
+  
+  //DEBUG_PRINT("update label block=%s\n",
+  //            (*handle)->fModeTipsBlock ? "TRUE" : "FALSE");
 
-  if (gEnableModeTips) {
-    CFStringRef allstr;
-    CFArrayRef array;
-
-    allstr = CFStringCreateWithCString(kCFAllocatorDefault,
-                                       inStr, kCFStringEncodingUTF8);
-    array = CFStringCreateArrayBySeparatingStrings(kCFAllocatorDefault,
-                                                   allstr, CFSTR("\n"));
-    /*
-    DEBUG_PRINT("UIMUpdatePropLabel() inStr='%s'\n", inStr);
-    if (array)
-      DEBUG_PRINT("UIMUpdatePropLabel() count=%d\n", CFArrayGetCount(array));
-    */
-
+  if (gEnableModeTips
+      && !(*handle)->fModeTipsBlock
+      ) {
+    CFStringRef allstr =
+      CFStringCreateWithCString(kCFAllocatorDefault,
+                                inStr, kCFStringEncodingUTF8);
+    if (!allstr) return;
+    
+    CFArrayRef array =
+      CFStringCreateArrayBySeparatingStrings(kCFAllocatorDefault,
+                                             allstr, CFSTR("\n"));
     if (array) {
       ShowModeTips(handle, array);
       CFRelease(array);
     }
-    CFRelease(allstr);
   }
 }
 
@@ -489,17 +520,16 @@ ParseHelperString(const char *str)
 
   if (!gActiveSession) return;
 
-
   if (strncmp("im_change", str, 9) == 0) {
     ParseIMChangeString(str);
   }
   else if (strncmp("prop_list_get", str, 13) == 0) {
-    //for (i = 0; i < gNumSession; i++)
-    uim_prop_list_update((*gActiveSession)->fUC);
+    if (gActiveProp)
+      uim_prop_list_update((*gActiveSession)->fUC);
   }
   else if (strncmp("prop_label_get", str, 14) == 0) {
-    //for (i = 0; i < gNumSession; i++)
-    uim_prop_label_update((*gActiveSession)->fUC);
+    if (gActiveProp)
+      uim_prop_label_update((*gActiveSession)->fUC);
   }
   else if (strncmp("prop_activate", str, 13) == 0) {
     CFMutableStringRef cfstr;
@@ -534,6 +564,8 @@ ParseHelperString(const char *str)
   else if (strncmp("prop_label_update", str, 17) == 0) {
   }
   else if (strncmp("focus_in", str, 8) == 0) {
+    gActiveProp = false;
+    //DEBUG_PRINT("deactivate prop\n");
   }
 }
 
@@ -579,7 +611,7 @@ UIMHelperDisconnect()
   gUimSock = NULL;
 
 #ifdef NEW_HELPER
-  gUimHelperConnected = FALSE;
+  gUimHelperConnected = false;
 #else
   gUimFD = -1;
 #endif
@@ -598,6 +630,7 @@ UIMHelperClose()
 #else
   if (gUimFD >= 0)
     uim_helper_close_client_fd(gUimFD);
+  gUimFD = -1;
 #endif
 }
 
@@ -605,22 +638,21 @@ UIMHelperClose()
 void
 UIMModeUpdate(void *inPtr, int inMode)
 {
-  //MUIMSessionHandle handle = (MUIMSessionHandle) inPtr;
+  MUIMSessionHandle handle = (MUIMSessionHandle) inPtr;
   //MUIMSessionHandle handle = gActiveSession;
+  
+  if (!handle || !(*handle)) return;
 
-#if DEBUG_MODE
-  DEBUG_PRINT("UIMModeUpdate() inMode=%d\n", inMode);
+#if DEBUG_SYNCMODE
+  DEBUG_PRINT("UIMModeUpdate() gMode=%d inMode=%d\n", gMode, inMode);
 #endif
 
   gMode = inMode;
-
-  /*
-  DEBUG_PRINT("MUIMSessionActivate() label update\n");
-  uim_prop_label_update((*handle)->fUC);
-  */
 }
 #endif
 
+#pragma mark -
+
 static void
 AddPreeditSegment(MUIMSessionHandle inHandle, int inAttr,
                   const char *inStr)
@@ -715,19 +747,13 @@ GetPreeditString(MUIMSessionHandle inHandle, UniCharPtr *outStr,
 }
 
 void
-CandClicked(UInt32 inRow)
+BlockUpdatePreedit()
 {
-  UInt32 idx = 0;
+  gBlockUpdatePreedit = true;
+}
 
-  idx = (*gActiveSession)->fDisplayLimit * (*gActiveSession)->fPageIndex + inRow;
-
-#if DEBUG_CANDIDATES
-  DEBUG_PRINT("CandClicked() inRow=%lu fPageIndex=%ld idx=%lu\n",
-              inRow, (*gActiveSession)->fPageIndex, idx);
-#endif
-
-  uim_set_candidate_index((*gActiveSession)->fUC, idx);
-
-  uim_press_key((*gActiveSession)->fUC, 'j', UMod_Control);
-  uim_release_key((*gActiveSession)->fUC, 'j', UMod_Control);
+void
+UnblockUpdatePreedit()
+{
+  gBlockUpdatePreedit = false;
 }
